# -*- coding: utf8 -*-

import rawes
from rawes.elastic_exception import ElasticException

from pyramid.settings import asbool

from zope.interface import Interface

class IRawES(Interface):
    pass

def _parse_settings(settings):

    rawes_args = {}
    defaults = {
        'url': 'localhost:9200',
        'timeout': 30,
        'except_on_error': False,
        'path': '',
        'connection_type': None
    }

    rawes_args = defaults.copy()

    # set string settings
    for short_key_name in ('url', 'path', 'connection_type'):
        key_name = 'rawes.%s' % (short_key_name,)
        if key_name in settings:
            rawes_args[short_key_name] = \
                settings.get(key_name, defaults.get(short_key_name))

    # boolean settings
    for short_key_name in ('except_on_error',):
        key_name = 'rawes.%s' % (short_key_name,)
        if key_name in settings:
            rawes_args[short_key_name] = \
                asbool(settings.get(key_name, defaults.get(short_key_name)))

    # integer settings
    for short_key_name in ('timeout',):
        key_name = 'rawes.%s' % (short_key_name,)
        if key_name in settings:
            rawes_args[short_key_name] = \
                int(settings.get(key_name, defaults.get(short_key_name)))

    return rawes_args

def _build_rawes(registry):
    """
    Build a RawES connection to Elastic Search and add it to the registry.
    """
    ES = registry.queryUtility(IRawES)
    if ES is not None:
        return ES

    settings = registry.settings
    rawes_args = _parse_settings(settings)

    ES = rawes.Elastic(rawes_args['url'], 
                       rawes_args['path'], 
                       rawes_args['timeout'],
                       rawes_args['connection_type'],
                       None,
                       rawes_args['except_on_error']
                       )

    registry.registerUtility(ES, IRawES)
    return registry.queryUtility(IRawES)

def get_rawes(registry):
    """
    Get the RawES connection.
    """
    return registry.queryUtility(IRawES)

def includeme(config):
    _build_rawes(config.registry)



# -*- coding: utf8 -*-

from pyramid import testing

from pyramid_rawes import (
    IRawES,
    get_rawes,
    _build_rawes,
    includeme,
    _parse_settings
    )

import rawes

try:
    import unittest2 as unittest
except ImportError:
    import unittest

class TestRegistry(object):

    def __init__(self, settings=None):

        if settings is None:
            self.settings = {}
        else:
            self.settings = settings

        self.rawes = None

    def queryUtility(self, iface):
        return self.rawes

    def registerUtility(self, rawes, iface):
        self.rawes = rawes

class TestGetAndBuild(unittest.TestCase):

    def test_get_rawes(self):
        r = TestRegistry()
        ES = rawes.Elastic('')
        r.registerUtility(ES, IRawES)
        ES2 = get_rawes(r)
        self.assertEquals(ES, ES2)

    def test_build_rawes_already_exists(self):
        r = TestRegistry()
        ES = rawes.Elastic('')
        r.registerUtility(ES, IRawES)
        ES2 = _build_rawes(r)
        self.assertEquals(ES, ES2)

    def test_build_rawes_default_settings(self):
        r = TestRegistry()
        ES = _build_rawes(r)
        self.assertIsInstance(ES, rawes.Elastic)

    def test_build_rawes_custom_settings(self):
        settings = {
            'rawes.url': 'elastic.search.org:9200',
            'rawes.path': '/search',
            'rawes.timeout': 123,
            'rawes.connection_type': 'http',
            'rawes.except_on_error': True
            }
        r = TestRegistry(settings)
        ES = _build_rawes(r)
        self.assertIsInstance(ES, rawes.Elastic)
        self.assertEquals('elastic.search.org:9200', ES.url)


class TestSettings(unittest.TestCase):

    def _assert_contains_all_keys(self, args):
        self.assertIn('url', args)
        self.assertIn('path', args)
        self.assertIn('connection_type', args)
        self.assertIn('except_on_error', args)
        self.assertIn('timeout', args)

    def test_get_default_settings(self):
        settings = {}
        args = _parse_settings(settings)
        self._assert_contains_all_keys(args)

    def test_get_some_settings(self):
        settings = {
            'rawes.url': 'elastic.search.org:9200',
            'rawes.timeout': 123,
            'rawes.except_on_error': True
            }
        args = _parse_settings(settings)
        self._assert_contains_all_keys(args)
        self.assertEquals('elastic.search.org:9200', args['url'])
        self.assertEquals(123, args['timeout'])
        self.assertEquals(True, args['except_on_error'])

    def test_get_all_settings(self):
        settings = {
            'rawes.url': 'elastic.search.org:9200',
            'rawes.path': '/search',
            'rawes.timeout': 123,
            'rawes.connection_type': 'http',
            'rawes.except_on_error': True
            }
        args = _parse_settings(settings)
        self._assert_contains_all_keys(args)
        self.assertEquals(123, args['timeout'])
        self.assertEquals(True, args['except_on_error'])

class TestIncludeMe(unittest.TestCase):

    def test_includeme(self):
        config = testing.setUp()
        config.registry.settings['rawes.url'] = 'localhost:9300'
        includeme(config)
        ES = config.registry.queryUtility(IRawES)
        self.assertIsInstance(ES, rawes.Elastic)
        self.assertEquals('localhost:9300', ES.url)
        



import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()
CHANGES = open(os.path.join(here, 'HISTORY.rst')).read()

requires = [
    'pyramid',
    'rawes'
    ]

tests_requires = [
    'nose',
    'coverage'
    ]

testing_extras = tests_requires + []

setup(name='pyramid_rawes',
      version='0.1.0',
      license='MIT',
      description='RawES binding for pyramid',
      long_description=README + '\n\n' +  CHANGES,
      classifiers=[
        'Intended Audience :: Developers',
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Framework :: Pyramid',
        'Topic :: Internet :: WWW/HTTP :: Indexing/Search'
        ],
      author='Koen Van Daele',
      author_email='koen_van_daele@telenet.be',
      url='',
      keywords='pyramid elastic search rawes',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      install_requires = requires,
      tests_require = tests_requires,
      extras_require = {
        'testing': testing_extras
        },
      test_suite='pyramid_rawes',
      )
